#!/usr/bin/env node
'use strict';

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _parser = require('./parser.js');

var _visitors = require('./visitors');

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const p = new _parser.Parser();

const pretty = process.argv.indexOf('--pretty-print');

if (~pretty) process.argv.splice(0, pretty, 1);

const input = _fs2.default.readFileSync(process.argv[2], 'utf8');

function getExerpt(input, location) {
  const lines = input.split(/\n/g);

  var marker = '  ~~';
  for (var i = 0; i < location.col - 1; ++i) marker += '~';

  return (location.line >= 3 ? '    ' + lines[location.line - 3] + '\n' : '') + (location.line >= 2 ? '    ' + lines[location.line - 2] + '\n' : '') + '    ' + lines[location.line - 1] + '\n' + _chalk2.default.white(marker) + _chalk2.default.green('^') + '\n' + (location.line < lines.length ? '    ' + lines[location.line] + '\n' : '') + (location.line + 1 < lines.length ? '    ' + lines[location.line + 1] + '\n' : '');
}

try {
  const res = p.parse(input);

  if (~pretty) {
    const pp = new _visitors.PrettyPrinter();
    console.log(res.accept(pp));
    process.exit(0);
  }

  const typer = new _visitors.Typer();
  const executor = new _visitors.Executor();

  res.accept(typer);
  res.accept(executor);
} catch (err) {
  var message = err.message;

  if (err.location) {
    message = _chalk2.default.white(process.argv[2]) + ': ligne ' + _chalk2.default.green(err.location.line) + ' col ' + _chalk2.default.white(err.location.col) + ': ' + message + '\n';
    message += getExerpt(input, err.location);
  }

  console.error(err.location ? message : err.stack);
}