'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
const $private = Symbol('private');

const token = Symbol('token');
const rule = Symbol('rule');
const epsilon = Symbol('epsilon');
const types = { [token]: token, [rule]: rule, [epsilon]: epsilon };

class LngSymbol {
  constructor(type = epsilon, name = '') {
    this[$private] = {};

    this.type = type;
    this.name = name;
  }

  get type() {
    return this[$private].type;
  }
  set type(type) {
    if (!(type in types)) throw new Error('Unknown type provided');
    this[$private].type = type;
  }

  formatedName() {
    return LngSymbol.formatedName(this.name);
  }

  toString() {
    switch (this.type) {
      case LngSymbol.token:
        return 'token(' + this.formatedName() + ')';
      case LngSymbol.rule:
        return 'rule(' + this.formatedName() + ')';
      case LngSymbol.epsilon:
        return 'epsilon';
    }
  }

  inspect() {
    return this.toString();
  }
}

exports.LngSymbol = LngSymbol;
LngSymbol.formatedName = function (name) {
  return name.replace('\n', '\\n').replace('\t', '\\t');
};

LngSymbol.token = token;
LngSymbol.rule = rule;
LngSymbol.epsilon = epsilon;