#!/usr/bin/env node

import fs from 'fs'
import {Parser} from './parser.js'
import {Executor, Typer, PrettyPrinter} from './visitors'
import chalk from 'chalk'

const p = new Parser()

const pretty = process.argv.indexOf('--pretty-print')

if (~pretty)
  process.argv.splice(0, pretty, 1)

const input = fs.readFileSync(process.argv[2], 'utf8')

function getExerpt(input, location) {
  const lines = input.split(/\n/g)

  var marker = '  ~~'
  for (var i = 0; i < location.col - 1; ++i)
    marker += '~'

  return (location.line >= 3 ? '    ' + lines[location.line - 3] + '\n' : '') +
    (location.line >= 2 ? '    ' + lines[location.line - 2] + '\n' : '') +
    '    ' + lines[location.line - 1] + '\n' +
    chalk.white(marker) + chalk.green('^') + '\n' +
    (location.line < lines.length ? '    ' + lines[location.line] + '\n' : '') +
    (location.line + 1 < lines.length ? '    ' + lines[location.line + 1] + '\n' : '')
}

try {
  const res = p.parse(input)

  if (~pretty) {
    const pp = new PrettyPrinter()
    console.log(res.accept(pp))
    process.exit(0)
  }

  const typer = new Typer()
  const executor = new Executor()

  res.accept(typer)
  res.accept(executor)
}
catch (err) {
  var message = err.message;

  if (err.location) {
    message = chalk.white(process.argv[2]) + ': ligne ' + chalk.green(err.location.line) + ' col ' + chalk.white(err.location.col) + ': ' + message + '\n'
    message += getExerpt(input, err.location)
  }

  console.error(err.location ? message : err.stack)
}
