import * as ast from "./ast"

import fs from 'fs'
import {Lexer as BaseLexer} from './lexer.js'
import {Parser as BaseParser} from './jungle/parser.js'
import {JungleError as Error} from './jungle/error.js'

export class Parser extends BaseParser {
  constructor() {
    super(fs.readFileSync(__dirname + '/../lib/parsing_tables.pt', 'utf8'), BaseLexer)

    this.registerCode("algo", 0, this.algo0);
    this.registerCode("algo_decls", 0, this.algo_decls0);
    this.registerCode("const_decls", 0, this.const_decls0);
    this.registerCode("const_decls", 1, this.const_decls1);
    this.registerCode("const_decls_list", 0, this.const_decls_list0);
    this.registerCode("const_decls_list", 1, this.const_decls_list1);
    this.registerCode("types_decls", 0, this.types_decls0);
    this.registerCode("types_decls", 1, this.types_decls1);
    this.registerCode("types_decls_list", 0, this.types_decls_list0);
    this.registerCode("types_decls_list", 1, this.types_decls_list1);
    this.registerCode("variables_decls", 0, this.variables_decls0);
    this.registerCode("variables_decls", 1, this.variables_decls1);
    this.registerCode("variable_decls_list", 0, this.variable_decls_list0);
    this.registerCode("variable_decls_list", 1, this.variable_decls_list1);
    this.registerCode("identifieur_list", 0, this.identifieur_list0);
    this.registerCode("identifieur_list", 1, this.identifieur_list1);
    this.registerCode("type_def", 0, this.type_def0);
    this.registerCode("type_def", 1, this.type_def1);
    this.registerCode("type_def", 2, this.type_def2);
    this.registerCode("type_def", 3, this.type_def3);
    this.registerCode("array_dimension_list", 0, this.array_dimension_list0);
    this.registerCode("array_dimension_list", 1, this.array_dimension_list1);
    this.registerCode("array_dimension_list", 2, this.array_dimension_list2);
    this.registerCode("array_dimension_list", 3, this.array_dimension_list3);
    this.registerCode("value", 0, this.value0);
    this.registerCode("value", 1, this.value1);
    this.registerCode("value", 2, this.value2);
    this.registerCode("value", 3, this.value3);
    this.registerCode("value", 4, this.value4);
    this.registerCode("value", 5, this.value5);
    this.registerCode("value", 6, this.value6);
    this.registerCode("value", 7, this.value7);
    this.registerCode("value", 8, this.value8);
    this.registerCode("routines_decls", 0, this.routines_decls0);
    this.registerCode("routines_decls", 1, this.routines_decls1);
    this.registerCode("routines_decls_no_empty", 0, this.routines_decls_no_empty0);
    this.registerCode("routines_decls_no_empty", 1, this.routines_decls_no_empty1);
    this.registerCode("routine_decl", 0, this.routine_decl0);
    this.registerCode("routine_decl", 1, this.routine_decl1);
    this.registerCode("routine_decls", 0, this.routine_decls0);
    this.registerCode("routine_params", 0, this.routine_params0);
    this.registerCode("routine_params", 1, this.routine_params1);
    this.registerCode("routine_params", 2, this.routine_params2);
    this.registerCode("routine_params", 3, this.routine_params3);
    this.registerCode("routine_params", 4, this.routine_params4);
    this.registerCode("local_params", 0, this.local_params0);
    this.registerCode("global_params", 0, this.global_params0);
    this.registerCode("algo_instructions", 0, this.algo_instructions0);
    this.registerCode("algo_instructions", 1, this.algo_instructions1);
    this.registerCode("algo_expressions_list", 0, this.algo_expressions_list0);
    this.registerCode("algo_expressions_list", 1, this.algo_expressions_list1);
    this.registerCode("expression", 0, this.expression0);
    this.registerCode("expression", 1, this.expression1);
    this.registerCode("valued_cplte_expression", 0, this.valued_cplte_expression0);
    this.registerCode("valued_cplte_expression", 1, this.valued_cplte_expression1);
    this.registerCode("valued_cplte_expression", 2, this.valued_cplte_expression2);
    this.registerCode("valued_cplte_expression", 3, this.valued_cplte_expression3);
    this.registerCode("valued_cplte_expression", 4, this.valued_cplte_expression4);
    this.registerCode("valued_cplte_expression", 5, this.valued_cplte_expression5);
    this.registerCode("valued_cplte_expression", 6, this.valued_cplte_expression6);
    this.registerCode("valued_cplte_expression", 7, this.valued_cplte_expression7);
    this.registerCode("valued_cplte_expression", 8, this.valued_cplte_expression8);
    this.registerCode("valued_cplte_expression", 9, this.valued_cplte_expression9);
    this.registerCode("valued_cplte_expression", 10, this.valued_cplte_expression10);
    this.registerCode("valued_cplte_expression", 11, this.valued_cplte_expression11);
    this.registerCode("valued_cplte_expression", 12, this.valued_cplte_expression12);
    this.registerCode("valued_cplte_expression", 13, this.valued_cplte_expression13);
    this.registerCode("valued_cplte_expression", 14, this.valued_cplte_expression14);
    this.registerCode("valued_cplte_expression", 15, this.valued_cplte_expression15);
    this.registerCode("valued_cplte_expression", 16, this.valued_cplte_expression16);
    this.registerCode("valued_cplte_expression", 17, this.valued_cplte_expression17);
    this.registerCode("valued_cplte_expression", 18, this.valued_cplte_expression18);
    this.registerCode("valued_cplte_expression", 19, this.valued_cplte_expression19);
    this.registerCode("valued_cplte_expression", 20, this.valued_cplte_expression20);
    this.registerCode("valued_cplte_expression", 21, this.valued_cplte_expression21);
    this.registerCode("valued_cplte_expression", 22, this.valued_cplte_expression22);
    this.registerCode("valued_cplte_expression", 23, this.valued_cplte_expression23);
    this.registerCode("valued_cplte_expression", 24, this.valued_cplte_expression24);
    this.registerCode("valued_cplte_expression", 25, this.valued_cplte_expression25);
    this.registerCode("valued_expression", 0, this.valued_expression0);
    this.registerCode("valued_expression", 1, this.valued_expression1);
    this.registerCode("valued_expression", 2, this.valued_expression2);
    this.registerCode("control_expression", 0, this.control_expression0);
    this.registerCode("control_expression", 1, this.control_expression1);
    this.registerCode("control_expression", 2, this.control_expression2);
    this.registerCode("control_expression", 3, this.control_expression3);
    this.registerCode("control_expression", 4, this.control_expression4);
    this.registerCode("valued_expression_list", 0, this.valued_expression_list0);
    this.registerCode("valued_expression_list", 1, this.valued_expression_list1);
    this.registerCode("valued_expression_list_not_empty", 0, this.valued_expression_list_not_empty0);
    this.registerCode("valued_expression_list_not_empty", 1, this.valued_expression_list_not_empty1);
    this.registerCode("sinon", 0, this.sinon0);
    this.registerCode("sinon", 1, this.sinon1);
    this.registerCode("ordre", 0, this.ordre0);
    this.registerCode("ordre", 1, this.ordre1);
    this.registerCode("selon_expressions", 0, this.selon_expressions0);
    this.registerCode("selon_expressions", 1, this.selon_expressions1);
    this.registerCode("autrement", 0, this.autrement0);
    this.registerCode("autrement", 1, this.autrement1);
    this.registerCode("scalar_list", 0, this.scalar_list0);
    this.registerCode("scalar_list", 1, this.scalar_list1);
    this.registerCode("scalar", 0, this.scalar0);
    this.registerCode("scalar", 1, this.scalar1);
    this.registerCode("scalar", 2, this.scalar2);
  }

  algo0(__arg_1__, __arg_2__, __arg_3__, __arg_4__, __arg_5__, __arg_6__, __arg_7__, __arg_8__, __arg_9__, __arg_10__, __arg_11__) {
    if (__arg_2__.value !== __arg_10__.value)
      throw new Error('Impossible de créer un algorithme avec deux noms différents : ' + __arg_2__.value + ' / ' + __arg_10__.value, __arg_10__.location);
    return new ast.Algorithm(__arg_1__.location, __arg_2__.value, __arg_4__.value, __arg_7__.value);
  }
  algo_decls0(__arg_1__, __arg_2__, __arg_3__, __arg_4__) {
    return new ast.AlgoDecls(__arg_1__.value, __arg_2__.value, __arg_3__.value, __arg_4__.value);
  }
  const_decls0(__arg_1__, __arg_2__, __arg_3__) {
    return __arg_3__.value;
  }
  const_decls1() {
    return [];
  }
  const_decls_list0(__arg_1__, __arg_2__, __arg_3__, __arg_4__, __arg_5__) {
    __arg_1__.value.push(new ast.Const(__arg_2__.location, __arg_2__.value, __arg_4__.value)); return __arg_1__.value;
  }
  const_decls_list1(__arg_1__, __arg_2__, __arg_3__, __arg_4__) {
    return [new ast.Const(__arg_1__.location, __arg_1__.value, __arg_3__.value)];
  }
  types_decls0(__arg_1__, __arg_2__, __arg_3__) {
    return __arg_3__.value;
  }
  types_decls1() {
    return [];
  }
  types_decls_list0(__arg_1__, __arg_2__, __arg_3__, __arg_4__, __arg_5__) {
    __arg_1__.value.push(__arg_4__.value.setLocation(__arg_2__.location).setName(__arg_2__.value)); return __arg_1__.value;
  }
  types_decls_list1(__arg_1__, __arg_2__, __arg_3__, __arg_4__) {
    return [__arg_3__.value.setLocation(__arg_1__.location).setName(__arg_1__.value)];
  }
  variables_decls0(__arg_1__, __arg_2__, __arg_3__) {
    return __arg_3__.value;
  }
  variables_decls1() {
    return [];
  }
  variable_decls_list0(__arg_1__, __arg_2__, __arg_3__, __arg_4__) {
    __arg_1__.value.push(new ast.Variables(__arg_2__.location, __arg_2__.value, __arg_3__.value)); return __arg_1__.value;
  }
  variable_decls_list1(__arg_1__, __arg_2__, __arg_3__) {
    return [new ast.Variables(__arg_1__.location, __arg_1__.value, __arg_2__.value)];
  }
  identifieur_list0(__arg_1__, __arg_2__, __arg_3__) {
    __arg_1__.value.push(__arg_3__.value); return __arg_1__.value;
  }
  identifieur_list1(__arg_1__) {
    return [__arg_1__.value];
  }
  type_def0(__arg_1__, __arg_2__, __arg_3__, __arg_4__, __arg_5__) {
    __arg_4__.value.unshift(__arg_2__.value); return new ast.Enum(__arg_4__.value);
  }
  type_def1(__arg_1__, __arg_2__) {
    return new ast.Array(__arg_2__.value, __arg_1__.value);
  }
  type_def2(__arg_1__, __arg_2__, __arg_3__, __arg_4__, __arg_5__, __arg_6__) {
    return new ast.Record(__arg_6__.value, __arg_3__.value);
  }
  type_def3(__arg_1__, __arg_2__) {
    return new ast.Pointer(__arg_2__.value);
  }
  array_dimension_list0(__arg_1__, __arg_2__, __arg_3__) {
    if (__arg_2__.value !== 'x') throw new Error('Invalid identifieur, expecting x', __arg_2__.location); __arg_1__.value.push(__arg_3__.value); return __arg_1__.value;
  }
  array_dimension_list1(__arg_1__, __arg_2__, __arg_3__) {
    if (__arg_2__.value !== 'x') throw new Error('Invalid identifieur, expecting x', __arg_2__.location); __arg_1__.value.push(__arg_3__.value); return __arg_1__.value;
  }
  array_dimension_list2(__arg_1__) {
    return [Number(__arg_1__.value)];
  }
  array_dimension_list3(__arg_1__) {
    return [__arg_1__.value];
  }
  value0(__arg_1__) {
    return new ast.Value(__arg_1__.location, 'entier', __arg_1__.value)
  }
  value1(__arg_1__, __arg_2__) {
    return new ast.Value(__arg_1__.location, 'entier', __arg_2__.value);
  }
  value2(__arg_1__, __arg_2__) {
    return new ast.Value(__arg_1__.location, 'entier', -__arg_2__.value);
  }
  value3(__arg_1__) {
    return new ast.Value(__arg_1__.location, 'reel', __arg_1__.value)
  }
  value4(__arg_1__, __arg_2__) {
    return new ast.Value(__arg_1__.location, 'reel', __arg_2__.value);
  }
  value5(__arg_1__, __arg_2__) {
    return new ast.Value(__arg_1__.location, 'reel', -__arg_2__.value);
  }
  value6(__arg_1__) {
    return new ast.Value(__arg_1__.location, 'chaine', __arg_1__.value)
  }
  value7(__arg_1__) {
    return new ast.Value(__arg_1__.location, 'caractere', __arg_1__.value)
  }
  value8(__arg_1__) {
    return new ast.Value(__arg_1__.location, '!nul')
  }
  routines_decls0(__arg_1__) {
    return __arg_1__.value;
  }
  routines_decls1() {
    return [];
  }
  routines_decls_no_empty0(__arg_1__, __arg_2__, __arg_3__) {
    __arg_1__.value.push(__arg_2__.value); return __arg_1__.value;
  }
  routines_decls_no_empty1(__arg_1__, __arg_2__) {
    return [__arg_1__.value];
  }
  routine_decl0(__arg_1__, __arg_2__, __arg_3__, __arg_4__, __arg_5__, __arg_6__, __arg_7__, __arg_8__, __arg_9__, __arg_10__, __arg_11__, __arg_12__) {
    if (__arg_3__.value != __arg_12__.value)
        throw new Error('Erreur de correspondance de nom pour la procédure : ' + __arg_3__.value + ' / ' + __arg_12__.value, __arg_12__.location);
      return new ast.Procedure(__arg_1__.location, __arg_3__.value, __arg_5__.value, __arg_8__.value);
  }
  routine_decl1(__arg_1__, __arg_2__, __arg_3__, __arg_4__, __arg_5__, __arg_6__, __arg_7__, __arg_8__, __arg_9__, __arg_10__, __arg_11__, __arg_12__, __arg_13__, __arg_14__) {
    if (__arg_3__.value != __arg_14__.value)
        throw new Error('Erreur de correspondance de nom pour la fonction ' + __arg_3__.value + ' / ' + __arg_14__.value, __arg_14__.location);
      return new ast.Function(__arg_1__.location, __arg_3__.value, __arg_5__.value, __arg_7__.value, __arg_10__.value);
  }
  routine_decls0(__arg_1__, __arg_2__, __arg_3__, __arg_4__) {
    return new ast.Routine(__arg_1__.value, __arg_2__.value, __arg_3__.value, __arg_4__.value)
  }
  routine_params0(__arg_1__, __arg_2__) {
    return new ast.Parameters(__arg_1__.value, __arg_2__.value);
  }
  routine_params1(__arg_1__, __arg_2__) {
    return new ast.Parameters(__arg_2__.value, __arg_1__.value, true);
  }
  routine_params2(__arg_1__) {
    return new ast.Parameters(__arg_1__.value, []);
  }
  routine_params3(__arg_1__) {
    return new ast.Parameters([], __arg_1__.value);
  }
  routine_params4() {
    return new ast.Parameters([], []);
  }
  local_params0(__arg_1__, __arg_2__, __arg_3__) {
    return __arg_3__.value;
  }
  global_params0(__arg_1__, __arg_2__, __arg_3__) {
    return __arg_3__.value;
  }
  algo_instructions0(__arg_1__) {
    return __arg_1__.value;
  }
  algo_instructions1() {
    return [];
  }
  algo_expressions_list0(__arg_1__, __arg_2__, __arg_3__) {
    __arg_1__.value.push(__arg_2__.value); return __arg_1__.value;
  }
  algo_expressions_list1(__arg_1__, __arg_2__) {
    return [__arg_1__.value]
  }
  expression0(__arg_1__) {
    return __arg_1__.value;
  }
  expression1(__arg_1__) {
    return __arg_1__.value;
  }
  valued_cplte_expression0(__arg_1__) {
    return __arg_1__.value;
  }
  valued_cplte_expression1(__arg_1__) {
    return new ast.IdentifierRef(__arg_1__.location, __arg_1__.value);
  }
  valued_cplte_expression2(__arg_1__, __arg_2__, __arg_3__, __arg_4__) {
    return new ast.RoutineCall(__arg_1__.location, __arg_1__.value, __arg_3__.value);
  }
  valued_cplte_expression3(__arg_1__, __arg_2__, __arg_3__, __arg_4__) {
    return new ast.ArrayAccess(__arg_2__.location, __arg_1__.value, __arg_3__.value);
  }
  valued_cplte_expression4(__arg_1__, __arg_2__, __arg_3__) {
    return new ast.Binary(__arg_2__.location, __arg_1__.value, '←', __arg_3__.value);
  }
  valued_cplte_expression5(__arg_1__, __arg_2__, __arg_3__) {
    return new ast.Binary(__arg_2__.location, __arg_1__.value, '+', __arg_3__.value);
  }
  valued_cplte_expression6(__arg_1__, __arg_2__, __arg_3__) {
    return new ast.Binary(__arg_2__.location, __arg_1__.value, '-', __arg_3__.value);
  }
  valued_cplte_expression7(__arg_1__, __arg_2__, __arg_3__) {
    return new ast.Binary(__arg_2__.location, __arg_1__.value, '=', __arg_3__.value);
  }
  valued_cplte_expression8(__arg_1__, __arg_2__, __arg_3__) {
    return new ast.Binary(__arg_2__.location, __arg_1__.value, '<>', __arg_3__.value);
  }
  valued_cplte_expression9(__arg_1__, __arg_2__, __arg_3__) {
    return new ast.Binary(__arg_2__.location, __arg_1__.value, 'et', __arg_3__.value);
  }
  valued_cplte_expression10(__arg_1__, __arg_2__, __arg_3__) {
    return new ast.Binary(__arg_2__.location, __arg_1__.value, 'ou', __arg_3__.value);
  }
  valued_cplte_expression11(__arg_1__, __arg_2__, __arg_3__) {
    return new ast.Binary(__arg_2__.location, __arg_1__.value, 'oue', __arg_3__.value);
  }
  valued_cplte_expression12(__arg_1__, __arg_2__, __arg_3__) {
    return new ast.Binary(__arg_2__.location, __arg_1__.value, '<', __arg_3__.value);
  }
  valued_cplte_expression13(__arg_1__, __arg_2__, __arg_3__) {
    return new ast.Binary(__arg_2__.location, __arg_1__.value, '<=', __arg_3__.value);
  }
  valued_cplte_expression14(__arg_1__, __arg_2__, __arg_3__) {
    return new ast.Binary(__arg_2__.location, __arg_1__.value, '>', __arg_3__.value);
  }
  valued_cplte_expression15(__arg_1__, __arg_2__, __arg_3__) {
    return new ast.Binary(__arg_2__.location, __arg_1__.value, '>=', __arg_3__.value);
  }
  valued_cplte_expression16(__arg_1__, __arg_2__, __arg_3__) {
    return new ast.Binary(__arg_2__.location, __arg_1__.value, '*', __arg_3__.value);
  }
  valued_cplte_expression17(__arg_1__, __arg_2__, __arg_3__) {
    return new ast.Binary(__arg_2__.location, __arg_1__.value, '/', __arg_3__.value);
  }
  valued_cplte_expression18(__arg_1__, __arg_2__, __arg_3__) {
    return new ast.Binary(__arg_2__.location, __arg_1__.value, 'div', __arg_3__.value);
  }
  valued_cplte_expression19(__arg_1__, __arg_2__, __arg_3__) {
    return new ast.Binary(__arg_2__.location, __arg_1__.value, 'mod', __arg_3__.value);
  }
  valued_cplte_expression20(__arg_1__, __arg_2__) {
    return new ast.PointerAccess(__arg_2__.location, __arg_1__.value);
  }
  valued_cplte_expression21(__arg_1__, __arg_2__, __arg_3__) {
    return new ast.RecordAccess(__arg_2__.location, __arg_1__.value, __arg_3__.value);
  }
  valued_cplte_expression22(__arg_1__, __arg_2__) {
    return new ast.Unary(__arg_1__.location, '+', __arg_2__.value);
  }
  valued_cplte_expression23(__arg_1__, __arg_2__) {
    return new ast.Unary(__arg_1__.location, '-', __arg_2__.value);
  }
  valued_cplte_expression24(__arg_1__, __arg_2__) {
    return new ast.Unary(__arg_1__.location, 'non', __arg_2__.value);
  }
  valued_cplte_expression25(__arg_1__, __arg_2__, __arg_3__) {
    return __arg_2__.value;
  }
  valued_expression0(__arg_1__) {
    return __arg_1__.value;
  }
  valued_expression1(__arg_1__) {
    return new ast.IdentifierRef(__arg_1__.location, __arg_1__.value);
  }
  valued_expression2(__arg_1__) {
    return __arg_1__.value
  }
  control_expression0(__arg_1__, __arg_2__, __arg_3__, __arg_4__, __arg_5__, __arg_6__, __arg_7__, __arg_8__) {
    return new ast.Si(__arg_1__.location, __arg_2__.value, __arg_5__.value, __arg_6__.value);
  }
  control_expression1(__arg_1__, __arg_2__, __arg_3__, __arg_4__, __arg_5__, __arg_6__, __arg_7__) {
    return new ast.TantQue(__arg_1__.location, __arg_2__.value, __arg_5__.value);
  }
  control_expression2(__arg_1__, __arg_2__, __arg_3__, __arg_4__, __arg_5__) {
    return new ast.TantQue(__arg_1__.location, __arg_5__.value, __arg_3__.value, true);
  }
  control_expression3(__arg_1__, __arg_2__, __arg_3__, __arg_4__, __arg_5__, __arg_6__, __arg_7__, __arg_8__, __arg_9__, __arg_10__, __arg_11__, __arg_12__) {
    return new ast.Pour(__arg_1__.location, __arg_2__.value, __arg_4__.value, __arg_6__.value, __arg_7__.value, __arg_10__.value);
  }
  control_expression4(__arg_1__, __arg_2__, __arg_3__, __arg_4__, __arg_5__, __arg_6__, __arg_7__, __arg_8__) {
    return new ast.Selon(__arg_1__.location, __arg_2__.value, __arg_5__.value, __arg_6__.value);
  }
  valued_expression_list0(__arg_1__) {
    return __arg_1__.value;
  }
  valued_expression_list1() {
    return [];
  }
  valued_expression_list_not_empty0(__arg_1__, __arg_2__, __arg_3__) {
    __arg_1__.value.push(__arg_3__.value); return __arg_1__.value;
  }
  valued_expression_list_not_empty1(__arg_1__) {
    return [__arg_1__.value];
  }
  sinon0(__arg_1__, __arg_2__, __arg_3__) {
    return __arg_3__.value;
  }
  sinon1() {
    return null;
  }
  ordre0(__arg_1__) {
    return true;
  }
  ordre1() {
    return false
  }
  selon_expressions0(__arg_1__, __arg_2__, __arg_3__, __arg_4__, __arg_5__) {
    __arg_1__.value.push(new ast.SelonExpression(__arg_3__.value, __arg_5__.value)); return __arg_1__.value;
  }
  selon_expressions1(__arg_1__, __arg_2__, __arg_3__, __arg_4__) {
    return [new ast.SelonExpression(__arg_2__.value, __arg_4__.value)];
  }
  autrement0(__arg_1__, __arg_2__) {
    return __arg_2__.value;
  }
  autrement1() {
    return null
  }
  scalar_list0(__arg_1__, __arg_2__, __arg_3__) {
    __arg_1__.value.push(__arg_3__.value); return __arg_1__.value;
  }
  scalar_list1(__arg_1__) {
    return [__arg_1__.value];
  }
  scalar0(__arg_1__) {
    return new ast.Scalar(__arg_1__.location, new ast.IdentifierRef(__arg_1__.location, __arg_1__.value));
  }
  scalar1(__arg_1__) {
    return new ast.Scalar(__arg_1__.location, new ast.Value(__arg_1__.location, 'entier', __arg_1__.value));
  }
  scalar2(__arg_1__) {
    return new ast.Scalar(__arg_1__.location, new ast.Value(__arg_1__.location, 'caractere', __arg_1__.value));
  }
}
