export class EmptyVisitor {
  array(arr) {
    arr.forEach(elt => elt.accept(this))
  }

  visitAlgorithm(algo) {
    algo.decls.accept(this)
    this.array(algo.instructions)
  }

  visitAlgoDecls(decls) {
    if (decls.consts.length)
      this.array(decls.consts)
    if (decls.types.length)
      this.array(decls.types)
    if (decls.variables.length)
      this.array(decls.variables)
    if (decls.routines && decls.routines.length)
      this.array(decls.routines)
  }

  visitConst(c) {
    c.value.accept(this)
  }

  visitEnum(e) {}

  visitArray(array) {}

  visitRecord(record) {
    this.array(record.properties)
  }

  visitVariables(variables) {}

  visitPointer(pointer) {}

  visitFunction(func) {
    func.decls.accept(this)
    this.array(func.instructions)
  }

  visitProcedure(func) {
    func.decls.accept(this)
    this.array(func.instructions)
  }

  visitRoutine(routine) {
    if (routine.params)
      routine.params.accept(this)
    this.visitAlgoDecls(routine)
  }

  visitParameters(parameters) {
    if (parameters.locals.length)
      this.array(parameters.locals)
    if (parameters.globals.length)
      this.array(parameters.globals)
  }

  visitBinary(binary) {
    binary.left.accept(this)
    binary.right.accept(this)
  }

  visitRecordAccess(record) {
    record.record.accept(this)
  }

  visitIdentifierRef(identifier) {}

  visitArrayAccess(array) {
    array.array.accept(this)
    this.array(array.positions)
  }

  visitUnary(unary) {
    unary.expr.accept(this)
  }

  visitValue(value) {}

  visitSi(si) {
    si.expr.accept(this)
    this.array(si.vrai)
    if (si.faux)
      this.array(si.faux)
  }

  visitRoutineCall(routine) {
    this.array(routine.args)
  }

  visitPour(pour) {
    pour.start.accept(this)
    pour.end.accept(this)
    this.array(pour.loop)
  }

  visitTantQue(tantque) {
    tantque.expr.accept(this)
    this.array(tantque.loop)
  }

  visitPointerAccess(pointer) {
    pointer.pointer.accept(this)
  }

  visitSelon(selon) {
    selon.test.accept(this)
    this.array(selon.expressions)

    if (selon.either && selon.either.length)
      this.array(selon.either)
  }

  visitSelonExpression(expr) {
    this.array(expr.scalars)
    this.array(expr.expressions)
  }

  visitScalar(scalar) {
    scalar.value.accept(this)
  }
}
